function [Z]=muestreo_rechazo(K,dx,X_M,U)
%Esta funcin entrega una secuencia de nmeros pseudo aleatorios $Z_n$ que 
% siguen una distribucin arbitraria $f(x)$ utilizando el mtodo del
%rechazo de muestra. 

%Las entradas de la funcin son las siguientes:
%K: nmero total de muestras.
%X_M: valor mximo a muestrear.
%dx: longitud del intervalo entre dos muestras.
%U: valor umbral de rechazo. Debe estar entre 0 y 1.

% En este ejemplo utilizamos:
% f(x)=x/gamma^2 *exp(-x^2/[2*gamma^2]), x>0
% g(x) = 1/[sqrt(2pi)*sigma] *exp(-(x-mu)^2/[2*sigma^2]

gamma=2;
sigma=2;
x=0:dx:X_M;

f=x./gamma^2.*exp(-x.^2/(2*gamma^2));
[~,i_max]=max(f);
%centramos la gaussiana en torno al valor mximo de g(x).
g=1/sqrt(2*pi*sigma^2)*exp(-(x-x(i_max)).^2/(2*sigma^2));
M=2;
figure(1)
plot(x,f,'b',x,M*g,'r');
xlabel('$x$');
legend ('$f(x)$','$g(x)$','Location','SouthOutside','Orientation','Horizontal');

%Generacin de muestras
t=0;
X=zeros(1,K);
Y=-1*ones(1,K);
figure(2),clf
for k=1:K
    %Aplicamos mtodo de inversa para producir muestra x_k a partir de
    %G^{-1}(x).
    V=rand;
    figure(2)
    subplot(1,2,1), scatter(V,0,'b');
    axis([0 1 -0.1 0.1])
    hold on
    %muestras desde g(x)$
    X(k)=norminv(V,x(i_max),sigma);
    %clculo de $f(x_k)$).
    if (X(k)>=0)
         f_k=X(k)/gamma^2*exp(-X(k)^2/(2*gamma^2));
    else
        f_k=0;
    end
    g_k=1/sqrt(2*pi*sigma^2)*exp(-(X(k)-x(i_max))^2/(2*sigma^2));
    %Aplicamos condicin de aceptacin
    if f_k/(M*g_k)>U
        t=t+1;
        Y(t)=X(k);
        figure(2)
        subplot(1,2,2), scatter(Y(t),0,'r');
        axis([0 X_M -0.1 0.1])
        hold on
    end 
    pause(0.01)
end
hold off
j_max=min(find(Y==-1));
%Se producen muestras.
Z=Y(1:j_max-1);
%calculo de histograma
bincenters=[dx/2:dx:X_M-dx/2]';
bins=length(bincenters);
h=zeros(bins,1);
for i=1:length(Z)
    for k=1:bins
        if Z(i)>bincenters(k)-dx/2 && X(i)<=bincenters(k)+dx/2
           h(k)=h(k)+1;
        end
    end
end

figure(3),clf
bar(bincenters,h/sum(h*dx),'r');
xlabel('$x$');
ylabel('$f(x)$');
hold on 
plot(x,f);
hold off
    